---------------------------------------------------------------
-- |
-- Module      : hmac_hu
-- Copyright   : (c) Hitesh Jasani, 2008
-- License     : BSD3
--
-- Maintainer  : Hitesh Jasani <hitesh.jasani@gmail.com>
-- Stability   : experimental
-- Portability : 
--
-- Created     : 2008-02-06
-- Version     : 0.0.1.0
--
-- hunit tests -- see RFC 2104 for test vectors
--
---------------------------------------------------------------

module Main where

import qualified Data.ByteString as B
import qualified Data.ByteString.Char8 as B8
import qualified Data.ByteString.Internal as BI
import Data.Digest.OpenSSL.HMAC
import Data.Word
import Test.HUnit


main :: IO Counts
main = runTestTT tests

tests :: Test
tests = TestList [ "md5 tests"         ~: hmac_tests safe_hmacs md5 md5_data
                 , "unsafe md5 tests"  ~: hmac_tests unsafe_hmacs md5 md5_data
                 ]


safe_hmacs
 :: CryptoHashFunction          -- ^ hashing function
 -> [Word8]                     -- ^ key
 -> String                      -- ^ message
 -> String                      -- ^ expected digest
 -> Assertion
safe_hmacs hf k m d = do
  digest <- hmac hf (B.pack k) (B8.pack m)
  d @=? digest

unsafe_hmacs
 :: CryptoHashFunction          -- ^ hashing function
 -> [Word8]                     -- ^ key
 -> String                      -- ^ message
 -> String                      -- ^ expected digest
 -> Assertion
unsafe_hmacs hf k m d =
  d @=? unsafeHMAC hf (B.pack k) (B8.pack m)


md5_data :: [(String, [Word8], String, String)]
md5_data = [ ( "no key"
             , []
             , "More text test vectors to stuff up EBCDIC machines :-)"
             , "e9139d1e6ee064ef8cf514fc7dc83e86")

           , ( "bin 0x0b key"
             , (replicate 16 0x0b)
             , "Hi There"
             , "9294727a3638bb1c13f48ef8158bfc9d")

           , ( "asc key"
             , (map BI.c2w "Jefe")
             , "what do ya want for nothing?"
             , "750c783e6ab0b503eaa86e310a5db738")

           , ( "bin 0xaa key, bin 0xdd msg"
             , (replicate 16 0xaa)
             , (map BI.w2c $ replicate 50 0xdd)
             , "56be34521d144c88dbb8c733f0e8b3f6")
           ]



hmac_tests :: (Testable t) => (a -> b -> c -> d -> t) -> a -> [(String, b, c, d)] -> Test
hmac_tests thmf hf = TestList . map (\(d,k,m,e) -> d ~: thmf hf k m e)

